@@ -26,7 +26,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-language-manager.c 5750 2007-07-27 10:40:00Z pborelli $
+ * $Id: gedit-language-manager.c 5855 2007-08-27 18:13:10Z paolo $
  */
 
 #include <string.h>
@@ -49,37 +49,7 @@ gedit_get_language_manager (void)
 	return language_manager;
 }
 
-GSList *
-remove_hidden_languages (GSList *languages)
-{
-	GSList *item;
-	GSList *prev;
-	GtkSourceLanguage *language;
 
-	prev = NULL;
-	item = languages;
-
-	while (item != NULL)
-	{
-		language = GTK_SOURCE_LANGUAGE(item->data);
-
-		if (gtk_source_language_get_hidden (language))
-		{
-			languages = g_slist_delete_link (languages, item);
-
-			/* Set item to start of the list if we removed the first
-			 * language */
-			item = prev == NULL ? languages : prev->next;
-		}
-		else
-		{
-			prev = item;
-			item = item->next;
-		}
-	}
-
-	return languages;
-}
 
 static gint
 language_compare (gconstpointer a, gconstpointer b)
@@ -96,12 +66,21 @@ GSList *
 gedit_language_manager_list_languages_sorted (GtkSourceLanguageManager *lm,
 					      gboolean                  include_hidden)
 {
-	GSList *languages;
+	GSList *languages = NULL;
+	const gchar * const *ids;
+	
+	ids = gtk_source_language_manager_get_language_ids (lm);
+	while (*ids != NULL)
+	{
+		GtkSourceLanguage *lang;
 
-	languages = gtk_source_language_manager_list_languages (lm);
+		lang = gtk_source_language_manager_get_language (lm, *ids);
+		g_return_val_if_fail (GTK_IS_SOURCE_LANGUAGE (lang), NULL);
+		++ids;
 
-	if (!include_hidden)
-		languages = remove_hidden_languages (languages);
+		if (include_hidden || !gtk_source_language_get_hidden (lang))
+			languages = g_slist_prepend (languages, lang);
+	}
 
 	return g_slist_sort (languages, (GCompareFunc)language_compare);
 }
@@ -159,8 +138,7 @@ GtkSourceLanguage *
 gedit_language_manager_get_language_from_mime_type (GtkSourceLanguageManager *lm,
 						    const gchar              *mime_type)
 {
-	GSList *languages;
-	GSList *l;
+	const gchar* const *languages;
 	GtkSourceLanguage *lang;
 	GtkSourceLanguage *parent = NULL;
 
@@ -176,15 +154,17 @@ gedit_language_manager_get_language_from_mime_type (GtkSourceLanguageManager *lm
 	gedit_debug_message (DEBUG_DOCUMENT,
 			     "Cache miss for %s", mime_type);
 
-	languages = gtk_source_language_manager_list_languages (lm);
+	languages = gtk_source_language_manager_get_language_ids (lm);
 
-	for (l = languages; l != NULL; l = l->next)
+	while (*languages != NULL)
 	{
 		gchar **mime_types;
 		gchar *found = NULL;
 		gint i;
 
-		lang = l->data;
+		lang = gtk_source_language_manager_get_language (lm, *languages);
+		g_return_val_if_fail (GTK_IS_SOURCE_LANGUAGE (lang), NULL);
+		++languages;
 
 		mime_types = gtk_source_language_get_mime_types (lang);
 		if (mime_types == NULL)
@@ -237,8 +217,6 @@ gedit_language_manager_get_language_from_mime_type (GtkSourceLanguageManager *lm
 		g_strfreev (mime_types);
 	}
 
-	g_slist_free (languages);
-
 	if (parent != NULL)
 		add_language_to_cache (lm, mime_type, parent);
 	
