@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window.c 5663 2007-06-29 08:52:15Z pborelli $
+ * $Id: gedit-window.c 5849 2007-08-26 10:27:57Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,6 +55,8 @@
 #include "gedit-plugins-engine.h"
 #include "gedit-enum-types.h"
 
+#define LANGUAGE_NONE (const gchar *)"LangNone"
+
 #define GEDIT_WINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object),\
 					 GEDIT_TYPE_WINDOW,                    \
 					 GeditWindowPrivate))
@@ -83,11 +85,6 @@ enum
 	TARGET_URI_LIST = 100
 };
 
-static const GtkTargetEntry drag_types[] =
-{
-	{ "text/uri-list", 0, TARGET_URI_LIST },
-};
-
 G_DEFINE_TYPE(GeditWindow, gedit_window, GTK_TYPE_WINDOW)
 
 static void	recent_manager_changed	(GtkRecentManager *manager,
@@ -125,12 +122,14 @@ gedit_window_dispose (GObject *object)
 	 */
 	gedit_plugins_engine_garbage_collect ();
 
-	if (window->priv->recent_manager != NULL)
+	if (window->priv->recents_handler_id != 0)
 	{
-		g_signal_handlers_disconnect_by_func (window->priv->recent_manager,
-						      G_CALLBACK (recent_manager_changed),
-						      window);
-		window->priv->recent_manager = NULL;
+		GtkRecentManager *recent_manager;
+
+		recent_manager =  gtk_recent_manager_get_default ();
+		g_signal_handler_disconnect (recent_manager,
+					     window->priv->recents_handler_id);
+		window->priv->recents_handler_id = 0;
 	}
 
 	if (window->priv->manager != NULL)
@@ -283,36 +282,6 @@ gedit_window_key_press_event (GtkWidget   *widget,
 	return handled;
 }
 
-static void
-gedit_window_screen_changed (GtkWidget *widget,
-			     GdkScreen *old_screen)
-{
-	GeditWindowPrivate *priv = GEDIT_WINDOW (widget)->priv;
-	GdkScreen *screen;
-
-	screen = gtk_widget_get_screen (widget);
-
-	/* FIXME: this seems to be happening when then spinner is destroyed!? */
-	if (old_screen == screen)
-		return;
-
-	if (old_screen != NULL)
-	{
-		g_signal_handlers_disconnect_by_func
-			(gtk_recent_manager_get_for_screen (old_screen),
-			 G_CALLBACK (recent_manager_changed), widget);
-	}
-
-	priv->recent_manager = gtk_recent_manager_get_for_screen (screen);
-	g_signal_connect (priv->recent_manager,
-			  "changed",
-			  G_CALLBACK (recent_manager_changed),
-			  widget);
-
-	if (GTK_WIDGET_CLASS (gedit_window_parent_class)->screen_changed)
-		GTK_WIDGET_CLASS (gedit_window_parent_class)->screen_changed (widget, old_screen);
-}
-
 static void
 gedit_window_tab_removed (GeditWindow *window,
 			  GeditTab    *tab) 
@@ -338,7 +307,6 @@ gedit_window_class_init (GeditWindowClass *klass)
 	widget_class->window_state_event = gedit_window_window_state_event;
 	widget_class->configure_event = gedit_window_configure_event;
 	widget_class->key_press_event = gedit_window_key_press_event;
-	widget_class->screen_changed = gedit_window_screen_changed;
 
 	signals[TAB_ADDED] =
 		g_signal_new ("tab_added",
@@ -622,7 +590,7 @@ set_paste_sensitivity_according_to_clipboard (GeditWindow  *window,
 	if (gdk_display_supports_selection_notification (display))
 	{
 		gtk_clipboard_request_contents (clipboard,
-						gdk_atom_intern ("TARGETS", FALSE),
+						gdk_atom_intern_static_string ("TARGETS"),
 						(GtkClipboardReceivedFunc) received_clipboard_contents,
 						g_object_ref (window));
 	}
@@ -799,8 +767,6 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 	gedit_plugins_engine_update_plugins_ui (window, FALSE);
 }
 
-#define GEDIT_SOURCE_LANGUAGE_KEY "sourcelanguage"
-
 static void
 language_toggled (GtkToggleAction *action,
 		  GeditWindow     *window)
@@ -816,16 +782,16 @@ language_toggled (GtkToggleAction *action,
 	if (doc == NULL)
 		return;
 
-	lang_id = g_object_get_data (G_OBJECT (action),
-				     GEDIT_SOURCE_LANGUAGE_KEY);
-	if (lang_id == NULL)
+	lang_id = gtk_action_get_name (GTK_ACTION (action));
+	
+	if (strcmp (lang_id, LANGUAGE_NONE) == 0)
 	{
 		/* Normal (no highlighting) */
 		lang = NULL;
 	}
 	else
 	{
-		lang = gtk_source_language_manager_get_language_by_id (
+		lang = gtk_source_language_manager_get_language (
 				gedit_get_language_manager (),
 				lang_id);
 		if (lang == NULL)
@@ -882,10 +848,16 @@ create_language_menu_item (GtkSourceLanguage *lang,
 
 	if (section_action == NULL)
 	{
+		gchar *section_name;
+		
+		section_name = gedit_utils_escape_underscores (section, -1);
+		
 		section_action = gtk_action_new (escaped_section,
-						 section,
+						 section_name,
 						 NULL,
 						 NULL);
+						 
+		g_free (section_name);
 
 		gtk_action_group_add_action (window->priv->languages_action_group,
 					     section_action);
@@ -894,31 +866,29 @@ create_language_menu_item (GtkSourceLanguage *lang,
 		gtk_ui_manager_add_ui (window->priv->manager,
 				       ui_id,
 				       "/MenuBar/ViewMenu/ViewHighlightModeMenu/LanguagesMenuPlaceholder",
-				       escaped_section, escaped_section,
+				       escaped_section,
+				       escaped_section,
 				       GTK_UI_MANAGER_MENU,
 				       FALSE);
 	}
 
 	/* now add the language item to the section */
 	lang_name = gtk_source_language_get_name (lang);
-	escaped_lang_name = g_markup_escape_text (lang_name, -1);
-
+	lang_id = gtk_source_language_get_id (lang);
+	
+	escaped_lang_name = gedit_utils_escape_underscores (lang_name, -1);
+	
 	tip = g_strdup_printf (_("Use %s highlight mode"), lang_name);
 	path = g_strdup_printf ("/MenuBar/ViewMenu/ViewHighlightModeMenu/LanguagesMenuPlaceholder/%s",
 				escaped_section);
 
-	action = gtk_radio_action_new (escaped_lang_name,
-				       lang_name,
+	action = gtk_radio_action_new (lang_id,
+				       escaped_lang_name,
 				       tip,
 				       NULL,
 				       index);
 
-	/* store lang id */
-	lang_id = gtk_source_language_get_id (lang);
-	g_object_set_data_full (G_OBJECT (action),
-				GEDIT_SOURCE_LANGUAGE_KEY,
-				g_strdup (lang_id),
-				(GDestroyNotify) g_free);
+	g_free (escaped_lang_name);
 
 	/* Action is added with a NULL accel to make the accel overridable */
 	gtk_action_group_add_action_with_accel (window->priv->languages_action_group,
@@ -928,7 +898,7 @@ create_language_menu_item (GtkSourceLanguage *lang,
 
 	/* add the action to the same radio group of the "Normal" action */
 	normal_action = gtk_action_group_get_action (window->priv->languages_action_group,
-						     "LangNone");
+						     LANGUAGE_NONE);
 	group = gtk_radio_action_get_group (GTK_RADIO_ACTION (normal_action));
 	gtk_radio_action_set_group (action, group);
 
@@ -940,13 +910,13 @@ create_language_menu_item (GtkSourceLanguage *lang,
 	gtk_ui_manager_add_ui (window->priv->manager,
 			       ui_id,
 			       path,
-			       escaped_lang_name, escaped_lang_name,
+			       lang_id, 
+			       lang_id,
 			       GTK_UI_MANAGER_MENUITEM,
 			       FALSE);
 
 	g_free (path);
 	g_free (tip);
-	g_free (escaped_lang_name);
 	g_free (escaped_section);
 }
 
@@ -965,7 +935,7 @@ create_languages_menu (GeditWindow *window)
 	
 	/* Translators: "None" means that no highlight mode is selected in the 
 	 * "View->Highlight Mode" submenu and so syntax highlighting is disabled */
-	action_none = gtk_radio_action_new ("LangNone", _("None"),
+	action_none = gtk_radio_action_new (LANGUAGE_NONE, _("None"),
 					    _("Disable syntax highlighting"),
 					    NULL,
 					    -1);
@@ -984,7 +954,8 @@ create_languages_menu (GeditWindow *window)
 	gtk_ui_manager_add_ui (window->priv->manager,
 			       id,
 			       "/MenuBar/ViewMenu/ViewHighlightModeMenu/LanguagesMenuPlaceholder",
-			       "LangNone", "LangNone",
+			       LANGUAGE_NONE, 
+			       LANGUAGE_NONE,
 			       GTK_UI_MANAGER_MENUITEM,
 			       TRUE);
 
@@ -992,7 +963,8 @@ create_languages_menu (GeditWindow *window)
 
 	/* now add all the known languages */
 	languages = gedit_language_manager_list_languages_sorted (
-						gedit_get_language_manager ());
+						gedit_get_language_manager (),
+						FALSE);
 
 	for (l = languages, i = 0; l != NULL; l = l->next, ++i)
 	{
@@ -1013,8 +985,7 @@ update_languages_menu (GeditWindow *window)
 	GList *l;
 	GtkAction *action;
 	GtkSourceLanguage *lang;
-	const gchar *lang_name;
-	gchar *escaped_lang_name;
+	const gchar *lang_id;
 
 	doc = gedit_window_get_active_document (window);
 	if (doc == NULL)
@@ -1022,11 +993,9 @@ update_languages_menu (GeditWindow *window)
 
 	lang = gedit_document_get_language (doc);
 	if (lang != NULL)
-		lang_name = gtk_source_language_get_name (lang);
+		lang_id = gtk_source_language_get_id (lang);
 	else
-		lang_name = "LangNone";
-
-	escaped_lang_name = g_markup_escape_text (lang_name, -1);
+		lang_id = LANGUAGE_NONE;
 
 	actions = gtk_action_group_list_actions (window->priv->languages_action_group);
 
@@ -1039,7 +1008,7 @@ update_languages_menu (GeditWindow *window)
 	}
 
 	action = gtk_action_group_get_action (window->priv->languages_action_group,
-					      escaped_lang_name);
+					      lang_id);
 
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
 
@@ -1051,7 +1020,6 @@ update_languages_menu (GeditWindow *window)
 	}
 
 	g_list_free (actions);
-	g_free (escaped_lang_name);
 }
 
 void
@@ -1059,6 +1027,7 @@ _gedit_recent_add (GeditWindow *window,
 		   const gchar *uri,
 		   const gchar *mime)
 {
+	GtkRecentManager *recent_manager;
 	GtkRecentData *recent_data;
 
 	static gchar *groups[2] = {
@@ -1066,6 +1035,8 @@ _gedit_recent_add (GeditWindow *window,
 		NULL
 	};
 
+	recent_manager =  gtk_recent_manager_get_default ();
+
 	recent_data = g_slice_new (GtkRecentData);
 
 	recent_data->display_name = NULL;
@@ -1076,7 +1047,7 @@ _gedit_recent_add (GeditWindow *window,
 	recent_data->groups = groups;
 	recent_data->is_private = FALSE;
 
-	gtk_recent_manager_add_full (window->priv->recent_manager,
+	gtk_recent_manager_add_full (recent_manager,
 				     uri,
 				     recent_data);
 
@@ -1089,9 +1060,11 @@ void
 _gedit_recent_remove (GeditWindow *window,
 		      const gchar *uri)
 {
-	gtk_recent_manager_remove_item (window->priv->recent_manager,
-					uri,
-					NULL);
+	GtkRecentManager *recent_manager;
+
+	recent_manager =  gtk_recent_manager_get_default ();
+
+	gtk_recent_manager_remove_item (recent_manager, uri, NULL);
 }
 
 static void
@@ -1154,8 +1127,6 @@ recent_manager_changed (GtkRecentManager *manager,
 	update_recent_files_menu (window);
 }
 
-
-
 /*
  * Manually construct the inline recents list in the File menu.
  * Hopefully gtk 2.12 will add support for it.
@@ -1164,6 +1135,7 @@ static void
 update_recent_files_menu (GeditWindow *window)
 {
 	GeditWindowPrivate *p = window->priv;
+	GtkRecentManager *recent_manager;
 	gint max_recents;
 	GList *actions, *l, *items;
 	GList *filtered_items = NULL;
@@ -1192,7 +1164,8 @@ update_recent_files_menu (GeditWindow *window)
 
 	p->recents_menu_ui_id = gtk_ui_manager_new_merge_id (p->manager);
 
-	items = gtk_recent_manager_get_items (p->recent_manager);
+	recent_manager =  gtk_recent_manager_get_default ();
+	items = gtk_recent_manager_get_items (recent_manager);
 
 	/* filter */
 	for (l = items; l != NULL; l = l->next)
@@ -1323,7 +1296,7 @@ create_menu_bar_and_toolbar (GeditWindow *window,
 	GtkUIManager *manager;
 	GtkWidget *menubar;
 	GtkToolItem *open_button;
-	GdkScreen *screen;
+	GtkRecentManager *recent_manager;
 	GtkRecentFilter *filter;
 	GError *error = NULL;
 	GeditLockdownMask lockdown;
@@ -1423,18 +1396,15 @@ create_menu_bar_and_toolbar (GeditWindow *window,
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	g_object_unref (action_group);
 
-	screen = gtk_widget_get_screen (GTK_WIDGET (window));
-	window->priv->recent_manager = gtk_recent_manager_get_for_screen (screen);
-
-	g_signal_connect (window->priv->recent_manager,
-			  "changed",
-			  G_CALLBACK (recent_manager_changed),
-			  window);
-
+	recent_manager = gtk_recent_manager_get_default ();
+	window->priv->recents_handler_id = g_signal_connect (recent_manager,
+							     "changed",
+							     G_CALLBACK (recent_manager_changed),
+							     window);
 	update_recent_files_menu (window);
 
 	/* recent files menu tool button */
-	window->priv->toolbar_recent_menu = gtk_recent_chooser_menu_new_for_manager (window->priv->recent_manager);
+	window->priv->toolbar_recent_menu = gtk_recent_chooser_menu_new_for_manager (recent_manager);
 
 	gtk_recent_chooser_set_local_only (GTK_RECENT_CHOOSER (window->priv->toolbar_recent_menu),
 					   FALSE);
@@ -1831,7 +1801,7 @@ update_cursor_position_statusbar (GtkTextBuffer *buffer,
 	gtk_text_iter_set_line_offset (&start, 0);
 	col = 0;
 
-	tab_size = gtk_source_view_get_tabs_width (GTK_SOURCE_VIEW (view));
+	tab_size = gtk_source_view_get_tab_width (GTK_SOURCE_VIEW (view));
 
 	while (!gtk_text_iter_equal (&start, &iter))
 	{
@@ -2279,6 +2249,45 @@ sync_name (GeditTab    *tab,
 	gedit_plugins_engine_update_plugins_ui (window, FALSE);
 }
 
+static GeditWindow *
+get_drop_window (GtkWidget *widget)
+{
+	GtkWidget *target_window;
+
+	target_window = gtk_widget_get_toplevel (widget);
+	g_return_val_if_fail (GEDIT_IS_WINDOW (target_window), NULL);
+
+	if ((GEDIT_WINDOW(target_window)->priv->state & GEDIT_WINDOW_STATE_SAVING_SESSION) != 0)
+		return NULL;
+	
+	return GEDIT_WINDOW (target_window);
+}
+
+static void
+load_uris_from_drop (GeditWindow  *window,
+		     gchar       **uri_list)
+{
+	GSList *uris = NULL;
+	gint i;
+	
+	if (uri_list == NULL)
+		return;
+	
+	for (i = 0; uri_list[i] != NULL; ++i)
+	{
+		uris = g_slist_prepend (uris, uri_list[i]);
+	}
+
+	uris = g_slist_reverse (uris);
+	gedit_commands_load_uris (window,
+				  uris,
+				  NULL,
+				  0);
+
+	g_slist_free (uris);
+}
+
+/* Handle drops on the GeditWindow */
 static void
 drag_data_received_cb (GtkWidget        *widget,
 		       GdkDragContext   *context,
@@ -2289,128 +2298,35 @@ drag_data_received_cb (GtkWidget        *widget,
 		       guint             time,
 		       gpointer          data)
 {
-	GtkWidget *target_window;
-	gchar **uris;
-	GSList *uri_list = NULL;
-	gint i;
+	GeditWindow *window;
+	gchar **uri_list;
 
-	if (info != TARGET_URI_LIST)
-		return;
-
-	g_return_if_fail (widget != NULL);
-
-	target_window = gtk_widget_get_toplevel (widget);
-	g_return_if_fail (GEDIT_IS_WINDOW (target_window));
+	window = get_drop_window (widget);
 	
-	if ((GEDIT_WINDOW(target_window)->priv->state & GEDIT_WINDOW_STATE_SAVING_SESSION) != 0)
+	if (window == NULL)
 		return;
 
-	uris = g_uri_list_extract_uris ((gchar *) selection_data->data);
-
-	for (i = 0; uris[i] != NULL; i++)
+	if (info == TARGET_URI_LIST)
 	{
-		gchar *uri;
-		
-		uri = gedit_utils_make_canonical_uri_from_shell_arg (uris[i]);
-		
-		/* Silently ignore malformed URI/filename */
-		if (uri != NULL)
-			uri_list = g_slist_prepend (uri_list, uri);	
+		uri_list = gedit_utils_drop_get_uris(selection_data);
+		load_uris_from_drop (window, uri_list);
+		g_strfreev (uri_list);
 	}
+}
 
-	g_strfreev (uris);
+/* Handle drops on the GeditView */
+static void
+drop_uris_cb (GtkWidget    *widget,
+	      gchar       **uri_list)
+{
+	GeditWindow *window;
 
-	if (uri_list == NULL)
+	window = get_drop_window (widget);
+	
+	if (window == NULL)
 		return;
 
-	uri_list = g_slist_reverse (uri_list);
-
-	gedit_commands_load_uris (GEDIT_WINDOW (target_window),
-				  uri_list,
-				  NULL,
-				  0);
-
-	g_slist_foreach (uri_list, (GFunc) g_free, NULL);	
-	g_slist_free (uri_list);
-}
-
-/*
- * Override the gtk_text_view_drag_motion and drag_drop 
- * functions to get URIs
- *
- * If the mime type is text/uri-list, then we will accept
- * the potential drop, or request the data (depending on the
- * function).
- *
- * If the drag context has any other mime type, then pass the
- * information onto the GtkTextView's standard handlers.
- * (widget_class->function_name).
- *
- * See bug #89881 for details
- */
-
-static gboolean
-drag_motion_cb (GtkWidget      *widget,
-		GdkDragContext *context,
-		gint            x,
-		gint            y,
-		guint           time)
-{
-	GtkTargetList *tl;
-	GtkWidgetClass *widget_class;
-	gboolean result;
-
-	tl = gtk_target_list_new (drag_types,
-				  G_N_ELEMENTS (drag_types));
-
-	/* If this is a URL, deal with it here, or pass to the text view */
-	if (gtk_drag_dest_find_target (widget, context, tl) != GDK_NONE) 
-	{
-		gdk_drag_status (context, context->suggested_action, time);
-		result = TRUE;
-	}
-	else
-	{
-		widget_class = GTK_WIDGET_GET_CLASS (widget);
-		result = (*widget_class->drag_motion) (widget, context, x, y, time);
-	}
-
-	gtk_target_list_unref (tl);
-
-	return result;
-}
-
-static gboolean
-drag_drop_cb (GtkWidget      *widget,
-	      GdkDragContext *context,
-	      gint            x,
-	      gint            y,
-	      guint           time)
-{
-	GtkTargetList *tl;
-	GtkWidgetClass *widget_class;
-	gboolean result;
-	GdkAtom target;
-
-	tl = gtk_target_list_new (drag_types,
-				  G_N_ELEMENTS (drag_types));
-
-	/* If this is a URL, just get the drag data */
-	target = gtk_drag_dest_find_target (widget, context, tl);
-	if (target != GDK_NONE)
-	{
-		gtk_drag_get_data (widget, context, target, time);
-		result = TRUE;
-	}
-	else
-	{
-		widget_class = GTK_WIDGET_GET_CLASS (widget);
-		result = (*widget_class->drag_drop) (widget, context, x, y, time);
-	}
-
-	gtk_target_list_unref (tl);
-
-	return result;
+	load_uris_from_drop (window, uri_list);
 }
 
 static void
@@ -2536,7 +2452,6 @@ notebook_tab_added (GeditNotebook *notebook,
 {
 	GeditView *view;
 	GeditDocument *doc;
-	GtkTargetList *tl;
 	GtkAction *action;
 
 	gedit_debug (DEBUG_WINDOW);
@@ -2605,29 +2520,9 @@ notebook_tab_added (GeditNotebook *notebook,
 
 	update_documents_list_menu (window);
 	
-	/* CHECK: it seems to me this does not work when tab are moved between
-	   windows */
-	/* Drag and drop support */
-	tl = gtk_drag_dest_get_target_list (GTK_WIDGET (view));
-	g_return_if_fail (tl != NULL);
-
-	gtk_target_list_add_table (tl, drag_types, G_N_ELEMENTS (drag_types));
-
 	g_signal_connect (view,
-			  "drag_data_received",
-			  G_CALLBACK (drag_data_received_cb), 
-			  NULL);
-
-	/* Get signals before the standard text view functions to deal 
-	 * with uris for text files.
-	 */
-	g_signal_connect (view,
-			  "drag_motion",
-			  G_CALLBACK (drag_motion_cb), 
-			  NULL);
-	g_signal_connect (view,
-			  "drag_drop",
-			  G_CALLBACK (drag_drop_cb), 
+			  "drop_uris",
+			  G_CALLBACK (drop_uris_cb), 
 			  NULL);
 
 	update_window_state (window);
@@ -2684,13 +2579,7 @@ notebook_tab_removed (GeditNotebook *notebook,
 					      G_CALLBACK (editable_changed),
 					      window);
 	g_signal_handlers_disconnect_by_func (view, 
-					      G_CALLBACK (drag_data_received_cb),
-					      NULL);
-	g_signal_handlers_disconnect_by_func (view, 
-					      G_CALLBACK (drag_motion_cb),
-					      NULL);
-	g_signal_handlers_disconnect_by_func (view, 
-					      G_CALLBACK (drag_drop_cb),
+					      G_CALLBACK (drop_uris_cb),
 					      NULL);
 
 	g_return_if_fail (window->priv->num_tabs >= 0);
@@ -3147,6 +3036,7 @@ gedit_window_init (GeditWindow *window)
 {
 	static gboolean is_first = TRUE;
 	GtkWidget *main_box;
+	GtkTargetList *tl;
 
 	gedit_debug (DEBUG_WINDOW);
 
@@ -3219,15 +3109,28 @@ gedit_window_init (GeditWindow *window)
 	gtk_widget_show (window->priv->hpaned);
 	gtk_widget_show (window->priv->vpaned);
 
-	/* Drag and drop support */
+	/* Drag and drop support, set targets to NULL because we add the
+	   default uri_targets below */
 	gtk_drag_dest_set (GTK_WIDGET (window),
 			   GTK_DEST_DEFAULT_MOTION |
 			   GTK_DEST_DEFAULT_HIGHLIGHT |
 			   GTK_DEST_DEFAULT_DROP,
-			   drag_types,
-			   G_N_ELEMENTS (drag_types),
+			   NULL,
+			   0,
 			   GDK_ACTION_COPY);
 
+	/* Add uri targets */
+	tl = gtk_drag_dest_get_target_list (GTK_WIDGET (window));
+	
+	if (tl == NULL)
+	{
+		tl = gtk_target_list_new (NULL, 0);
+		gtk_drag_dest_set_target_list (GTK_WIDGET (window), tl);
+		gtk_target_list_unref (tl);
+	}
+	
+	gtk_target_list_add_uri_targets (tl, TARGET_URI_LIST);
+
 	/* Connect signals */
 	g_signal_connect (window->priv->notebook,
 			  "switch_page",
